%% Copyright (C) 2018 Simone Balmelli and Francesco Moresino
%%
%% This file is part of PrefShare.
%%
%% PrefShare is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% PrefShare is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with PrefShare. If not, see <http://www.gnu.org/licenses/>.

%% -*- texinfo -*-
%% @deftypefn {Function File} {@var{s} =} dialog(@var{s_def})
%% 
%% Provide a user interface to define the values of
%% the parameter settings @var{s} (see @code{initsettings}).
%% In particular, call the routine @code{parameters_balanced_exp}
%% to calculate questionnaire split parameters that are
%% as close as possible to the user's wish.
%% 
%% Input:
%% Structure @var{s_def} containing the default settings;
%% @var{s_def} is initialized by @code{initsettings} and can
%% contain empty fields.
%% 
%% Output:
%% Structure @var{s} containing all parameter settings,
%% which may be equal to those given in @var{s_def}, or
%% may be modified or defined from scratch by the user.
%% 
%% @seealso{initsettings, parameters_balanced_exp}
%%
%% Programmed by Simone Balmelli (2018).
%% @end deftypefn

function s=dialog(sdef)

s = initsettings();

%% ============== use/don't use default settings ===========
%
[sel ok] = listdlg("ListString",...
                     {"Use the default settings",...
                      "Modify the default settings",...
                      "Enter new settings"
                     },...
                    "PromptString",...
                     {"Choose among the following options:"},...
                     "Name", "Input box",... 
                    "SelectionMode","Single","ListSize",[250,60]);

% If "Cancel" button is selected
if ok==0
	error("Evaluation interrupted.\r\n")
end

if sel==1 
	s = sdef;
	return;
else if sel==3
	sdef.pars = [];
	sdef.v = [];
	sdef.k = [];
	sdef.b = [];
	sdef.mFit = [];
	sdef.concprofs = [];
end

% Convert default settings to cell strings
sdefc = cell(6,1);
sdefc{1} = sprintf("%i," , sdef.pars);
if length(sdefc{1} >0)
	sdefc{1} = sdefc{1}(1:end-1); % remove last cell
end
sdefc{2} = num2str(sdef.b);
sdefc{3} = num2str(sdef.k);
sdefc{4} = num2str(sdef.v);
sdefc{5} = num2str(sdef.mFit);

sdefc{6} = ""; % initialize competition as empty string
for i=1:size(sdef.concprofs,1)
	sdefc{5+i} = sprintf("%i," , sdef.concprofs(i,:));
	if length(sdefc{5+i} >0)
		sdefc{5+i} = sdefc{5+i}(1:end-1); % remove last cell
	end
end

%% ============== attributes and levels ====================
% 
prompt={"Enter the number of levels for each attribute,\
         \r\nseparated by a comma:"};          

answers = inputdlg(prompt,"Input box",1,sdefc(1));

% if "Cancel" button is selected
if size(answers,1)==0
	error("Evaluation interrupted.\r\n")
end

check_pars = str2num(answers{1}); 

% allow input correction
while !(isvector(check_pars) && size(check_pars,1)==1)
  corr_prompt={"Please correct:\
               \r\nEnter the number of levels for each attribute,\
               \r\nseparated by a comma:"};  
  corr_answers = inputdlg(corr_prompt,"Input box",1,answers);

  % if "Cancel" button is selected
  if size(corr_answers,1)==0
   error("Evaluation interrupted.\r\n")
  end
  check_pars = str2num(corr_answers{1});
end

s.pars = check_pars;

%% =========== competition profile number ================
%
if size(sdef.concprofs,1)==0
	ncdef_cell = {""};
else
	ncdef_cell = {num2str(size(sdef.concprofs,1))};
end

prompt = {"Enter the number of competition's profiles"};
answers = inputdlg(prompt,"Input box",1,ncdef_cell);

% if "Cancel" button is selected
if size(answers,1)==0
	error("Evaluation interrupted.\r\n")
end

nc = str2num(answers{1});
s.concprofs = zeros(nc,length(s.pars));

%% ========= enter competition's profiles ==================
%
if nc!=(size(sdefc)-5)
	sdefc(6:5+nc) = cell(nc,1); % empty default competition profs
end

for i=1:nc
	prompt = {sprintf("Enter competition's profile n. %i,\r\n\
with attributes separated by commas", i)};
	answers = inputdlg(prompt,"Input box",1,sdefc(5+i));

	% if "Cancel" button is selected
	if size(answers,1)==0
		error("Evaluation interrupted.\r\n")
	end

	check_conc = str2num(answers{1});

	% allow input correction
	while !(isvector(check_conc) && size(check_conc)==[1 length(s.pars)])
	  corr_prompt={"Please correct:\
	               \r\nEnter the number of levels for each attribute,\
	               \r\nseparated by a comma:"};  
	  corr_answers = inputdlg(corr_prompt,"Input box",1,answers{1});

      % if "Cancel" button is selected
      if size(corr_answers,1)==0
      	error("Evaluation interrupted.\r\n")
      end

	  check_conc = str2num(corr_answers{1});
	end

	s.concprofs(i,:) = check_conc;
end

%% ============== limit on v  =========================
%
msg = "Continue without imposing a maximal limit to the \
number of regression profiles?";
choice = questdlg(msg,"Input box","Continue","Impose limit","Cancel","Continue");

if strcmp(choice,"Cancel")
	error("Evaluation interrupted.\r\n")
elseif strcmp(choice,"Continue")
	V = prod(s.pars) - nc; % total number of profiles minus
                           % number of competition profs.
elseif strcmp(choice,"Impose limit")
	prompt = {"Enter the maximal limit to the\
\r\nnumber of regression profiles:"};

	answers = inputdlg(prompt,"Input box");

	% if "Cancel" button is selected
	if size(answers,1)==0
		error("Evaluation interrupted.\r\n")
	end

	V_input = str2num(answers{1});
	V = min((prod(s.pars)-nc),V_input);
end

%% ============== wished parameters ==================
%
% user's wish
prompt={"Wished number of respondents:",...
        "Wished number of cards per respondent:"
        };
answers = inputdlg(prompt,"Input box",1,sdefc(2:3));  

% if "Cancel" button is selected
if size(answers,1)==0
	error("Evaluation interrupted.\r\n")
end

b_wish = str2num(answers{1});
k_wish = str2num(answers{2});

%% ============== propose parameters ==================
%
choice="No";
first=1;
while strcmp(choice,"No")

	if first
		answers= {"1","1"};
	else
		prompt = {"Enter importance weights\r\n\
for the following parameters:\r\n\r\nweight for the number of respondents",
 "weight for the number of cards per respondent"};
		answers = inputdlg(prompt,"Input box",1,answers);

		% if "Cancel" button is selected
		if size(answers,1)==0
			error("Evaluation interrupted.\r\n")
		end
	end
	first = 0;

	b_weight = str2num(answers{1});
	k_weight = str2num(answers{2});
	v_weight = "1";
	[k b v r] = parameters_balanced_exp(k_wish,b_wish,...
                V,k_weight,b_weight,v_weight);
	
	msg = 	sprintf("Do you agree with the following parameters?\r\n\r\n\
number of respondents: %i\r\n\
number of cards per respondent: %i\r\n\r\n", b, k);
	        
	choice = questdlg(msg,"Input box");         

	% if "Cancel" button is selected
	if strcmp(choice,"Cancel")
		error("Evaluation interrupted.\r\n")
	end
end

s.b = b;
s.k = k;
s.v = v;
s.r = r;
	
%% ========= enter number of mFed tries ==================
%
prompt = {"Enter the number of independent tries\r\n\
of the algorithm for D-optimality:"};

answers = inputdlg(prompt,"Input box",1,sdefc(5));

% if "Cancel" button is selected
if size(answers,1)==0
	error("Evaluation interrupted.\r\n")
end

s.mFit = str2num(answers{1});

end  
