%% Copyright (C) 2018 Simone Balmelli and Francesco Moresino
%%
%% This file is part of PrefShare.
%%
%% PrefShare is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% PrefShare is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with PrefShare. If not, see <http://www.gnu.org/licenses/>.

%% -*- texinfo -*-
%% @deftypefn {Function File} {[@var{Xnew} @var{Dnew} @var{inds_new}] =} mFed (@var{X}, @var{inds})
%% @deftypefnx {Function File} {[@var{Xnew} @var{Dnew} @var{inds_new}] =} mFed (@var{X}, @var{inds}, @var{it})
%% 
%% Given an initial regression matrix @var{X},
%% perform the modified Fedorov algorithm
%% to maximize the determinant of @var{X}'*@var{X}.
%% 
%% The optimal profile exchanges are found by taking
%% the maximal elements in the ouptut of @code{Delta_xj}.
%% 
%% If no further improvement is found during an iteration, stop.
%% If the argument @var{it} is specified,
%% stop at the latest after @var{it} iterations. 
%%
%% @var{FD}, the full array of possible profiles, is constructed as a global variable 
%% within the scope of @code{Run}, to which this function belongs.
%% 
%% Input args:
%% Regression matrix @var{X};
%% Indices @var{inds} of the profiles belonging to @var{X}
%% (calculated as the row number of the corresponding profiles in @var{FD});
%% Maximal number @var{it} of iterations.
%% 
%% Output:
%% Resulting "best" regression matrix @var{Xnew}, in dummy-coded form;
%% Matrix @var{Dnew} = inv(@var{Xnew}'*@var{Xnew});
%% Indices @var{inds_new} of the profiles belonging to @var{Xnew}
%% (calculated as the row number of the corresponding profiles in @var{FD});
%% 
%% Reference(s): Cook and Nachtsheim, Technometrics 22.3 (1980).
%% 
%% @seealso{Delta_xj, mFed_it}
%%
%% Programmed by Simone Balmelli (2018).
%% @end deftypefn

function [Xnew Dnew inds_new] = mFed(X, inds, varargin)

global FD;

% manage maximal number of iterations
if isempty(varargin)
  itmax = Inf;
elseif length(varargin)==1
  itmax = varargin{1};
else
  error('too many arguments')
end  

v = size(X,1);
D = inv(X'*X);

% variables controlling the 'while' loop
[goon, i] = deal(1);

% iterate the algorithm
while goon && i<=itmax 
  detold = det(D);
  perm = randperm(v);
  goon=0;
  
  % make the exchanges
  for j=1:v
    [delta_max, ind] = max(Delta_xj(perm(j),X,D));
    
    % make an exchange if it increases delta 
    % and if no treatment gets replicated 
    if delta_max > eps && sum(ind==inds)==0 
      X(perm(j),:) = FD(ind,:);
      inds(perm(j)) = ind;
      
      % if at least one exchange has been made, continue
      goon=1;
    end
    D = inv(X'*X);
  end
  i = i+1;
  
  % if the determinant has not changed, stop
  if abs(det(D) - detold) < eps
    goon=0;
  end  

end
  Xnew = X;
  Dnew = D;
  inds_new = inds;

end  
