%% Copyright (C) 2018 Simone Balmelli and Francesco Moresino
%%
%% This file is part of PrefShare.
%%
%% PrefShare is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% PrefShare is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with PrefShare. If not, see <http://www.gnu.org/licenses/>.

%% -*- texinfo -*-
%% @deftypefn {Function File} {@var{Ddes} =} mFed_it (@var{v}, @var{pars}, @var{it})
%% 
%% Perform @var{it} tries of the modified Fedorov algorithm @code{mFed},
%% and select the best design among the obtained ones.
%% 
%% Every try is initialized by a random design (see @code{randD}).
%%
%% @var{FD}, the full array of possible profiles, is constructed as a global variable 
%% within the scope of @code{Run}, to which this function belongs.
%% 
%% Input args: 
%% Number @var{v} of profiles in the D-optimal design;
%% Array @var{pars} of design parameters
%% (see field @code{pars} in @code{initsettings.m});
%% Number @var{it} of iterations.
%% 
%% Output: matrix @var{Ddes} representing the D-optimal
%% (or nearly D-optimal) design obtained with the modified Fedorov algorithm.
%% The returned design matrix @var{Ddes} is in factorial form, with level labels starting from 1.
%% 
%% Reference(s): Cook and Nachtsheim, Technometrics 22.3 (1980).
%% 
%% @seealso{mFed}
%%
%% Programmed by Simone Balmelli (2018).
%% @end deftypefn

function Ddes = mFed_it(v,pars,it)

global FD;

% initialize
bestgm =0;
gms = 0;
p = size(FD,2);
bestX = [];

% main part
for i=1:it
  [XR, inds] = randD(v,FD);
  [X, D, inds] = mFed(XR,inds);
  gms(i) = nthroot(det(D),p);
  
  if gms(i) < bestgm || i==1 
    bestX = X;
    bestgm = gms(i);
  end
  
  fprintf('running mFed: step %i / %i\r',i,it)
  fflush(stdout);
end

fprintf('running mFed: step %i / %i\r\n',it,it)
fflush(stdout);

% sort rows in canonical order
[uX sort_idx] = sortrows(undummy(bestX,pars));
bestX = bestX(sort_idx,:);

Ddes = undummy(bestX,pars);
end
