%% Copyright (C) 2018 Simone Balmelli and Francesco Moresino
%%
%% This file is part of PrefShare.
%%
%% PrefShare is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% PrefShare is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with PrefShare. If not, see <http://www.gnu.org/licenses/>.

%% -*- texinfo -*-
%% @deftypefn {Function File} {[@var{kk} @var{bb} @var{vv} @var{rr} ] =} parameters_balanced_exp (@var{K},@var{B},@var{V},@var{alpha_k},@var{alpha_b},@var{alpha_v})
%% 
%%  Given @var{K} (desired number of profiles addressed to each respondent), 
%%  @var{B} (desired number of respondents),
%%  and @var{V} (upper limit to the number of profiles),
%%  this program finds integers  @var{kk}, @var{bb}, @var{vv} and @var{rr} such that
%%  1) @var{vv} =< @var{V} is as big as possible,
%%  2) @var{kk} and @var{bb} are as close as possible to @var{K} and @var{B}, and 
%%  3) @var{vv} * @var{r} = @var{bb} * @var{kk}.
%%  The performance criterion for requirements 1) and 2) is
%%  max( @var{alpha_v}*(@var{V}-@var{vv})/@var{V}, @var{alpha_k}*|@var{kk}-@var{K}|/@var{K},  @var{alpha_b}*|@var{bb}-@var{B}|/@var{B} ). 
%% 
%% This function is called by the user interface dialogue @code{dialog},
%% in order to help the user selecting a meaningful set of experimental parameters. 
%%
%%  Input args: 
%%  Desired number @var{K} of profiles administered to each respondent;
%%  Desired number @var{B} of respondents;
%%  Upper limit @var{V} to the number of profiles;
%%  Weight @var{alpha_k} for the @var{K}-criterion;
%%  Weight @var{alpha_b} for the @var{B}-criterion;
%%  Weight @var{alpha_v} for the @var{V}-criterion.
%% 
%%  Output:
%%  Actual number @var{kk} of profiles administered to each respondent;
%%  Actual number @var{bb} of respondents for the experiment;
%%  Number @var{vv} of regression profiles;
%%  Number @var{rr} of profile replications in the questionnaire.
%%
%% @seealso{dialog}
%%
%% Programmed by Francesco Moresino (2018).
%% @end deftypefn


function [kk, bb, vv, rr] = parameters_balanced_exp(K, B, V,alpha_k, alpha_b, alpha_v)


if(K>=B)
  error('Please choose k<b');
endif
if(K>=V)
  error('Please choose k<v');
endif

r1=floor(K*B/V);
r2=ceil(K*B/V);


if (r1==0)
  rr=1;
  kk=K;
  bb=B;
  vv=K*B;
elseif (r1==r2)
  rr=r1;
  kk=K;
  bb=B;
  vv=V;
 else
  stop=0;
  n=1;
  delta = inf;
  delta_2 = inf;
  while (stop==0)
    k_min=K-n;
    k_max=K+n;
    epsilon=(n+1)/K;
	b_min=ceil(B*(1-epsilon*alpha_k/alpha_b));
    if b_min<0 b_min=0; endif
    b_max=floor(B*(1+epsilon*alpha_k/alpha_b));
    v_min=ceil(V*(1-epsilon*alpha_k/alpha_v));
    if v_min<0 v_min=0; endif
    for l=k_min:k_max
      for c=b_min:b_max
        for w=v_min:V
        
          if (r1*w==l*c)
            Delta=max([alpha_v*(V-w)/V, alpha_k*abs((l-K)/K), alpha_b*abs((c-B)/B)]);
            Delta_2=alpha_v*(V-w)/V + alpha_k*abs((l-K)/K) + alpha_b*abs((c-B)/B);
            if ((Delta < delta) || (Delta== delta && Delta_2 < delta_2))
              delta=Delta;
              delta_2=Delta_2;
              rr=r1;
              kk=l;
              bb=c;
              vv=w;
              stop=1;
            endif %Delta < delta
          endif %r1*w=l*c
          
          if (r2*w==l*c)
            Delta=max([alpha_v*(V-w)/V, alpha_k*abs((l-K)/K), alpha_b*abs((c-B)/B)]);
            Delta_2=alpha_v*(V-w)/V + alpha_k*abs((l-K)/K) + alpha_b*abs((c-B)/B);
            if ((Delta < delta) || (Delta== delta && Delta_2 < delta_2))
              delta=Delta;
              delta_2=Delta_2;
              rr=r2;
              kk=l;
              bb=c;
              vv=w;
              stop=1;
            endif %Delta < delta
          endif %r2*w=l*c
          
        endfor %w
      endfor %c
    endfor %l
    n=n+1;;
  endwhile 
endif


if(vv*rr != bb*kk)
  rr
  kk
  bb
  vv
  error('Suitable parameters not found (equality v*r=b*k is violated).');
endif
if(vv>V)
  vv
  error('Suitable parameters not found (constraint v < max(v) is violated).');
endif

end
