%% Copyright (C) 2018 Simone Balmelli and Francesco Moresino
%%
%% This file is part of PrefShare.
%%
%% PrefShare is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% PrefShare is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with PrefShare. If not, see <http://www.gnu.org/licenses/>.

%% -*- texinfo -*-
%% @deftypefn {Function File} {@var{BD} =} randBD (@var{v}, @var{k}, @var{b})
%% @deftypefnx {Function File} {@var{BD} =} randBD (@var{v}, @var{k}, @var{b}, @var{itmax})
%% 
%% 
%% Construct a random block design with @var{v} profiles (or "treatments"),
%% @var{k} profiles per block and @var{b} blocks.
%% 
%% No two replications of the same profile are allowed 
%% to stay in the same block.
%% 
%% If no design is found, the search is restarted; 
%% this is done at most @var{itmax} times, 
%% after which a failure message is printed.
%% 
%% Input args: 
%% Number @var{v} of different profiles;
%% Number @var{k} of profiles per block;
%% Number @var{b} of blocks;
%% Maximal number @var{itmax} of tries;
%% if the function is called with 3 arguments,
%% @example
%% @var{itmax} = 1000
%% @end example 
%% by default.
%% 
%% Output: 
%% Block design @var{BD} (rows indicate blocks, elements indicate profiles)
%% of the random block design.
%%
%% Reference(s): This algorithm is based on Nguyen, Technometrics 36.3 (1994).
%%
%% @seealso{Run}
%%
%% Programmed by Simone Balmelli (2018).
%% @end deftypefn

function BD = randBD(v,k,b,varargin)

% manage args in
if nargin > 4
  error('Too many args in')
elseif nargin ==4
  maxit = varargin{1};    
elseif nargin ==3
  maxit = 1000;
end  

if mod(b*k,v)~=0
  error('v must be a divisor of k*b')
elseif k > v
  error('there are more cards for each respondents than profiles')
end  

r = b*k/v;

% if the block design is complete
if v==k
  BD = repmat(1:v,b,1);
  return
end

% loop control parameters
restart = 1;
it = 0;

while restart

  exitloop = 0; % exit for loop over rep
  
  % raise error if max. number of iterations is reached
  if it==maxit
      error('failed')   
  end
  it = it+1;
  
  % ---- initialize ---- %
  % bxk matrix: elements are the treats. contained in each b;
  BK = zeros(b,k);
  
  % vxr matrix: elements are the blocks containing each v;
  VR = zeros(v,r);  
  
  abs = ones(1,b);  % available blocks
  nfbs = abs;       % non-full blocks
  % -------------------- % 
  
  % run over the r treatment replications
  for rep=1:r

    % assign a single replication of treatment i
    % to an available block
    for i=1:v    
    
      % random block among available ones   
      bidx = find(abs)(randperm(sum(abs),1));
      
      % put treatment at the first empty place in the block  
      BK(bidx,find(~BK(bidx,:))(1))=i;    
      
      % correspondingly modify VR
      VR(i,rep)=bidx;
      
      % if the block is full, remove it from nfbs;
      if length(find(BK(bidx,:)))==k
        nfbs(bidx)=0;
      end      

      % available blocks for next i
      abs = nfbs;      
      if i<v   
        abs(nonzeros(VR(i+1,:)))=0;
      elseif rep<r
        abs(nonzeros(VR(1,:)))=0;
      else
        restart=0;        
        continue;  % design is complete  
      end 
      
      % there may be no place for the remaining replications
      % in that case, restart algorithm from the beginning
      if sum(abs)<r-rep+1
        exitloop = 1;
        break;
      end
      
    end
    
    % exit loop over rep in order to restart algorithm
    if exitloop
      break;
    end  
    
  end
end

BD = sort(BK,2);  
end  
